'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global log, ElementTypes, destroyElement, parseUri, browser, adblockBegin */

const elementPurger = {
  onPurgeRequest(request, sender, sendResponse) {
    if (document &&
        document.location &&
        document.location.href &&
        request.command === 'purge-elements' &&
        request.frameUrl === document.location.href.replace(/#.*$/, '')) {
      elementPurger.purgeElements(request);
      sendResponse({});
    }
  },

  // Remove elements on the page of |request.elType| that request
  // |request.url|.  Will try again if none are found unless |lastTry|.
  purgeElements(request, lastTry) {
    const { elType } = request;
    const { url } = request;

    log('[DEBUG]', 'Purging:', lastTry, elType, url);

    const tags = {};
    tags[ElementTypes.image] = { IMG: 1 };
    tags[ElementTypes.subdocument] = { IFRAME: 1, FRAME: 1 };
    tags[ElementTypes.object] = { OBJECT: 1, EMBED: 1 };

    const srcdata = this.srcsFor(url);
    for (let i = 0; i < srcdata.length; i += 1) {
      for (const tag in tags[elType]) {
        const src = srcdata[i];
        const attr = (tag === 'OBJECT' ? 'data' : 'src');
        const selector = `${tag}[${attr}${src.op}"${src.text}"]`;

        const results = document.querySelectorAll(selector);
        log('[DEBUG]', '  ', results.length, 'results for selector:', selector);
        if (results.length) {
          for (let j = 0; j < results.length; j += 1) {
            destroyElement(results[j], elType);
          }

          return; // I doubt the same URL was loaded via 2 different src attrs.
        }
      }
    }

    // No match; try later.  We may still miss it (race condition) in which
    // case we give up, rather than polling every second or waiting 10 secs
    // and causing a jarring page re-layout.
    if (!lastTry) {
      const that = this;
      setTimeout(() => {
        that.purgeElements(request, true);
      }, 2000);
    }
  },

  // Return a list of { op, text }, where op is a CSS selector operator and
  // text is the text to select in a src attr, in order to match an element on
  // this page that could request the given absolute |url|.
  srcsFor(url) {
    // NB: <img src="a#b"> causes a request for "a", not "a#b".  I'm
    // intentionally ignoring IMG tags that uselessly specify a fragment.
    // AdBlock will fail to hide them after blocking the image.
    const urlParts = parseUri(url);
    const pageParts = this.pageLocation;
    const results = [];

    // Case 1: absolute (of the form "abc://de.f/ghi" or "//de.f/ghi")
    if (url) {
      const urlMatch = url.match(/:(\/\/.*)$/);
      if (urlMatch) {
        results.push({ op: '$=', text: urlMatch[1] });
      }
    }
    if (urlParts.hostname === pageParts.hostname) {
      const urlSearchAndHash = urlParts.search + urlParts.hash;

      // Case 2: The kind that starts with '/'
      results.push({ op: '=', text: urlParts.pathname + urlSearchAndHash });

      // Case 3: Relative URL (of the form "ab.cd", "./ab.cd", "../ab.cd" and
      // "./../ab.cd")
      const pageDirs = pageParts.pathname.split('/');
      const urlDirs = urlParts.pathname.split('/');
      let i = 0;
      while (pageDirs[i] === urlDirs[i]
             && i < pageDirs.length - 1
             && i < urlDirs.length - 1) {
        i += 1; // i is set to first differing position
      }

      const dir = new Array(pageDirs.length - i).join('../');
      const path = urlDirs.slice(i).join('/') + urlSearchAndHash;
      if (dir) {
        results.push({ op: '$=', text: dir + path });
      } else {
        results.push({ op: '=', text: path });
        results.push({ op: '=', text: `./${path}` });
      }
    }

    return results;
  },

  // To enable testing
  pageLocation: document.location,
};

adblockBegin({
  startPurger() {
    browser.runtime.onMessage.addListener(elementPurger.onPurgeRequest);
  },

  stopPurger() {
    browser.runtime.onMessage.removeListener(elementPurger.onPurgeRequest);
  },
});
