'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global translate, parseUri, localizePage, browser, log,
splitMessageWithReplacementText */

// get the list of subscribed filters and
// all unsubscribed default filters
const BG = browser.extension.getBackgroundPage();
const unsubscribedDefaultFilters = [];
const subscribedFilterNames = [];
const enabledSettings = [];
let malwareDomains = {};
let tabId;

// Get debug info
let debugInfo;
BG.getDebugInfo((info) => {
  debugInfo = info;
});

const args = parseUri.parseSearch(document.location.search);

// generate the URL to the issue tracker
function generateReportURL() {
  let result = 'https://adblock.tenderapp.com/discussion/new' +
      '?category_id=ad-report&discussion[private]=1&discussion[title]=';

  let domain = '<enter URL of webpage here>';

  if (args && args.url) {
    domain = parseUri(args.url).hostname;
  }
  result += encodeURIComponent(`Ad report: ${domain}`);

  const body = [];
  let count = 1;
  body.push('Last step -- point me to the ad so I can fix the bug! ' +
  "Don't leave anything out or I'll probably " +
  'have to ignore your report. Thanks!');
  body.push('');
  body.push('Also, if you can put your name (or a screen name) ' +
  'and a contact email access in the boxes above, that would be great!');
  body.push('');
  body.push('We need the email so that we can contact you if we need more information ' +
  'than what you give us in your report. Otherwise, we might not be able to fix it.');
  body.push('');
  if (!args || !args.url) {
    body.push(`${count}. Paste the URL of the webpage showing an ad: `);
    body.push('');
    body.push('');
    count += 1;
  }

  body.push(`${count}. Exactly where on that page is the ad? What does it ` +
  'look like? Attach a screenshot, with the ad clearly marked, ' +
  'if you can.');
  body.push('');
  body.push('');
  count += 1;
  body.push(`${count}. If you have created the filter which removes reported ad, please paste it here: `);
  body.push('');
  body.push('');
  count += 1;
  body.push(`${count}. Any other information that would be helpful, besides ` +
  'what is listed below: ');
  body.push('');
  body.push('');
  body.push("-------- Please don't touch below this line. ---------");
  if (args && args.url) {
    body.push('=== URL with ad ===');
    body.push(args.url);
    body.push('');
  }

  body.push(debugInfo);
  body.push('');
  body.push('=== Question Responses ===');
  const answers = $('span[class="answer"]');
  const text = $('div[class="section"]:visible');
  for (let i = 0, n = 1; (i < answers.length, i < text.length); i += 1, n += 1) {
    body.push(`${n}.${text[i].id}: ${answers[i].getAttribute('chosen')}`);
  }

  body.push('');

  result = `${result}&discussion[body]=${encodeURIComponent(body.join('  \n'))}`; // Two spaces for Markdown newlines

  return result;
}

const createReadableReport = function (data) {
  const body = [];
  if (data.location) {
    body.push('* Location of ad *');
    body.push(data.location);
  }

  if (data.expect) {
    body.push('');
    body.push('* Working Filter? *');
    body.push(data.expect);
  }

  body.push('');

  // Get written debug info
  // data.debug is the debug info object
  const content = [];
  content.push('* Debug Info *');
  content.push('');
  if (data.debug &&
        data.debug.filter_lists) {
    content.push(data.debug.filter_lists);
  }

  content.push('');
  // Custom & Excluded filters might not always be in the object
  if (data.custom_filters) {
    content.push('=== Custom Filters ===');
    content.push(data.debug.custom_filters);
    content.push('');
  }

  if (data.exclude_filters) {
    content.push('=== Exclude Filters ===');
    content.push(data.debug.exclude_filters);
    content.push('');
  }

  if (data.debug &&
        data.debug.settings) {
    content.push('=== Settings ===');
    content.push(data.debug.settings);
  }

  content.push('');
  if (data.debug &&
        data.debug.other_info) {
    content.push('=== Other Info ===');
    content.push(data.debug.other_info);
  }

  body.push(content.join('\n'));
  body.push('');
  return body.join('\n');
}; // end of createReadableReport

// Pretty Print the data
const prepareManualReport = function (data, status, HTTPerror, respObj) {
  const body = [];
  body.push(createReadableReport(data));
  if (status) {
    body.push(`Status: ${status}`);
  }

  if (HTTPerror) {
    body.push(`HTTP error code: ${HTTPerror}`);
  }

  if (respObj) {
    body.push(`Server error information: ${JSON.stringify(respObj)}`);
  }

  $('#manual_submission')
    .val(body.join('\n'));
};

function sendReport() {
  // Cache access to input boxes
  const $name = $('#step_report_name');
  const $email = $('#step_report_email');
  const $location = $('#step_report_location');
  const $filter = $('#step_report_filter');
  let problems = 0;
  // Reset any error messages
  $email.removeClass('inputError');
  $name.removeClass('inputError');
  // Reset any error messages
  $('#screen_capture_file_label')
    .css('color', 'black');
  $('#step_response_error')
    .parent()
    .fadeOut();
  $('#step_response_success')
    .parent()
    .fadeOut();
  $('#adreport_missing_info')
    .hide();
  // Validate user entered info
  if ($name.val() === '') {
    problems += 1;
    $name.addClass('inputError');
    $('#adreport_missing_info')
      .show();
  }

  if ($email.val() === '' ||
      $email.val()
        .search(/^.+@.+\..+$/) === -1) {
    problems += 1;
    $email.addClass('inputError');
    $('#adreport_missing_info')
      .show();
  }

  if ($('#screen_capture_file')[0].files.length === 0) {
    $('#adreport_missing_screenshot')
      .show();
    problems += 1;
    $('#screen_capture_file_label')
      .css('color', '#f00');
  }

  if (problems) {
    $('html, body')
      .animate({
        scrollTop: $('#adreport_missing_info')
          .offset()
          .top,
      }, 2000);
    return;
  }

  const reportData = {
    title: 'Ad Report',
    name: $name.val(),
    email: $email.val(),
    location: $location.val(),
    filter: $filter.val(),
    debug: debugInfo,
    noscreencapturefile: true,
    url: '',
  };
  let domain = '';
  domain = parseUri(args.url).hostname.replace(/((http|https):\/\/)?(www.)?/g, '');
  reportData.title = `${reportData.title}: ${domain}`;
  reportData.url = args.url;
  const theAnswers = [];
  const answers = $('span[class="answer"]');
  const text = $('div[class="section"]:visible');
  const minArrayLength = Math.min(answers.length, text.length);
  for (let i = 0; i < minArrayLength; i += 1) {
    theAnswers.push(`${i + 1}.${text[i].id}: ${answers[i].getAttribute('chosen')}`);
  }

  reportData.answers = theAnswers.join('\n');

  // Handle any HTTP or server errors
  const handleResponseError = function (respObj) {
    $('#step_response_error')
      .parent()
      .fadeIn();
    if (respObj &&
          Object.prototype.hasOwnProperty.call(respObj, 'error_msg')) {
      $('#step_response_error_msg')
        .text(translate(respObj.error_msg));
    }
    // re-enable the button(s) if the error is recoverable (the user can re-submit)
    if (respObj &&
          Object.prototype.hasOwnProperty.call(respObj, 'retry_allowed') &&
          respObj.retry_allowed === 'true') {
      $('#step_report_submit')
        .prop('disabled', false);
      $('#step_response_error_manual_submission')
        .hide();
    } else {
      $('#step_response_error_manual_submission a')
        .attr('href', 'https://adblocksupport.freshdesk.com/support/tickets/new');
      $('#step_response_error_manual_submission a')
        .attr('target', '_blank');
      $('#step_response_error_manual_submission')
        .show();
    }

    $('html, body')
      .animate({
        scrollTop: $('#step_response_error')
          .offset()
          .top,
      }, 2000);
  };

  $('#debug-info').val(createReadableReport(reportData));

  const sendData = function () {
    const formdata = new FormData();
    formdata.append('ad_report', JSON.stringify(reportData));
    const uploadFileElement = $('#screen_capture_file');
    if (uploadFileElement && (uploadFileElement.length > 0)
      && uploadFileElement[0].files.length > 0) {
      formdata.append('screencapturefile', $('#screen_capture_file')[0].files[0]);
    }

    $('#debug-info')
      .val(createReadableReport(reportData));
    $.ajax({
      jsonp: false,
      url: 'https://getadblock.com/freshdesk/adReport.php',
      data: formdata,
      contentType: false,
      processData: false,
      success(respText) {
        $('#step_report_submit')
          .prop('disabled', true);
        // if a ticket was created, the response should contain a ticket id #
        if (respText) {
          try {
            const respObj = JSON.parse(respText);

            if (respObj &&
                      Object.prototype.hasOwnProperty.call(respObj, 'helpdesk_ticket') &&
                      Object.prototype.hasOwnProperty.call(respObj.helpdesk_ticket, 'display_id')) {
              $('#step_report_submit')
                .prop('disabled', true);
              // if a ticket was created, the response should contain a ticket id #
              $('#step_response_success')
                .parent()
                .fadeIn();
              $('html, body')
                .animate({
                  scrollTop: $('#step_response_success')
                    .offset()
                    .top,
                }, 2000);
            } else {
              prepareManualReport(reportData, null, null, respObj);
              handleResponseError(respObj);
            }
          } catch (e) {
            prepareManualReport(reportData);
            handleResponseError();
          }
        } else {
          prepareManualReport(reportData);
          handleResponseError();
        }
      },

      error(xhrInfo, status, HTTPerror) {
        prepareManualReport(reportData, status, HTTPerror);
        handleResponseError();
      },

      type: 'POST',
    });
  };

  // Retrieve extension info
  const askUserToGatherExtensionInfo = function () {
    reportData.extensions = 'Extension information not available in Firefox';
    sendData();
  }; // end of askUserToGatherExtensionInfo

  if (browser &&
      browser.tabs &&
      browser.tabs.detectLanguage) {
    let tabIdInt = -1;
    try {
      tabIdInt = parseInt(tabId, 10);
    } catch (e) {
      reportData.language = 'unknown';
      askUserToGatherExtensionInfo();
      return;
    }

    browser.tabs.detectLanguage(tabIdInt, (language) => {
      if (language) {
        reportData.language = language;
      }

      askUserToGatherExtensionInfo();
    }); // end of detectLanguage
  } else {
    reportData.language = 'unknown';
    askUserToGatherExtensionInfo();
  }
} // end of sendReport()

// Check every domain of downloaded resource against malware-known domains
const checkForMalware = function () {
  const tabFrameData = BG.getFrameData(args.tabId);
  if (!tabFrameData) {
    return;
  }

  const frames = [];
  const loadedResources = [];
  const extractedDomains = [];
  let infected = null;

  // Get all loaded frames
  for (const object in tabFrameData) {
    if (object && !Number.isNaN(object)) {
      frames.push(object);
    }
  }
  // Push loaded resources from each frame into an array
  for (let i = 0; i < frames.length; i += 1) {
    if (tabFrameData[frames[i]].resources
      && Object.keys(tabFrameData[frames[i]].resources).length !== 0) {
      loadedResources.push(tabFrameData[frames[i]].resources);
    }
  }

  // Extract domains from loaded resources
  for (let i = 0; i < loadedResources.length; i += 1) {
    for (const key in loadedResources[i]) {
      // Push just domains, which are not already in extractedDomains array
      const resource = key.split(':|:');
      if (resource &&
            resource.length === 2 &&
            extractedDomains.indexOf(parseUri(resource[1]).hostname) === -1) {
        extractedDomains.push(parseUri(resource[1]).hostname);
      }
    }
  }

  // Compare domains of loaded resources with domain.json
  for (let i = 0; i < extractedDomains.length; i += 1) {
    if (malwareDomains && malwareDomains.adware.indexOf(extractedDomains[i]) > -1) {
      // User is probably infected by some kind of malware,
      // because resource has been downloaded from malware/adware/spyware site.
      infected = true;
    }
  }

  $('.gifloader').hide();
  if (infected) {
    $('#step_update_filters_DIV').hide();
    $('#malwarewarning').text(translate('malwarewarning'));
    $('a', '#malwarewarning').attr('href', 'https://help.getadblock.com/support/solutions/articles/6000055822-i-m-seeing-similar-ads-on-every-website-');
  } else {
    $('#step_update_filters_DIV').show();
    $('#malwarewarning').text(translate('malwarenotfound'));
  }

  $('#malwarewarning').show();
};

// Auto-scroll to bottom of the page
$('input[type=radio]').click((event) => {
  event.preventDefault();
  $('html, body').animate({ scrollTop: $(document).height() }, 'slow');
});

// Auto-scroll to bottom of the page
$('select').change((event) => {
  event.preventDefault();
  $('html, body').animate({ scrollTop: $(document).height() }, 'slow');
});

$(document).ready(() => {
  localizePage();

  let subs = BG.getSubscriptionsMinusText();
  for (const id in subs) {
    if (!subs[id].subscribed && !subs[id].userSubmitted) {
      unsubscribedDefaultFilters[id] = subs[id];
    } else if (subs[id].subscribed) {
      subscribedFilterNames.push(id);
    }
  }

  const settings = BG.getSettings();
  for (const setting in settings) {
    if (settings[setting]) {
      enabledSettings.push(setting);
    }
  }

  // Shows the instructions for how to enable all extensions according to the browser of the user
  $('.chrome_only').hide();

  // Sort the languages list
  const languageOptions = $('#step_language_lang option');
  const currentlySelectedOption = $('#step_language_lang > option[selected]');
  languageOptions.sort((a, b) => {
    if (!a.text) {
      return -1;
    }
    if (!b.text) {
      return 1;
    } // First one is empty
    if (!a.value) {
      return 1;
    }
    if (!b.value) {
      return -1;
    } // 'Other' at the end
    if (a.getAttribute('i18n') === 'lang_english') {
      return -1;
    } // English second
    if (b.getAttribute('i18n') === 'lang_english') {
      return 1;
    }
    return (a.text > b.text) ? 1 : -1;
  });

  $('#step_language_lang').empty().append(languageOptions);
  $('#step_language_lang').val(currentlySelectedOption.val());

  ({ tabId } = args);
  // STEP 1: Malware/adware detection
  // Fetch file with malware-known domains
  malwareDomains = BG.getMalwareDomains();
  // Check, if downloaded resources are available,
  // if not, just reload tab with parsed tabId
  if (BG.getSettings().show_advanced_options) {
    checkForMalware();
  } else {
    browser.runtime.onMessage.addListener((request, sender, sendResponse) => {
      if (request && request.command === 'reloadcomplete') {
        BG.setSetting('show_advanced_options', false);
        checkForMalware();
        sendResponse({});
      }
    });

    BG.reloadTab(tabId);
  }

  // STEP 2: update filters
  // Updating the users filters
  $('#UpdateFilters').click(function updateFiltersClick() {
    $(this).attr('disabled', 'disabled');
    BG.updateSubscriptionsNow();
    $('.afterFilterUpdate input').removeAttr('disabled');
    $('.afterFilterUpdate').removeClass('afterFilterUpdate');
  });
  // if the user clicks a radio button
  $('#step_update_filters_no').click(() => {
    $('#step_update_filters').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'no');
    const textNode = document.createTextNode(translate('no'));
    newSpan.appendChild(textNode);
    $('#step_update_filters').append(newSpan);
    $('#checkupdate').text(translate('adalreadyblocked'));
  });

  $('#step_update_filters_yes').click(() => {
    $('#step_update_filters').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'yes');
    const textNode = document.createTextNode(translate('yes'));
    newSpan.appendChild(textNode);

    $('#step_update_filters').append(newSpan);

    subs = BG.getSubscriptionsMinusText();
    // if the user is subscribed to Acceptable-Ads, ask them to disable it
    if (subs && subs.acceptable_ads && subs.acceptable_ads.subscribed) {
      $('#step_update_aa_DIV')
        .show();
      $('.odd')
        .css('background-color', '#f8f8f8');
    } else {
      $('#step_disable_extensions_DIV').fadeIn().css('display', 'block');
      $('.even')
        .css('background-color', '#f8f8f8');
    }
  });

  // STEP 3: disable AA - IF enabled...

  $('#DisableAA')
    .click(function disableAAClick() {
      $(this)
        .prop('disabled', true);
      BG.unsubscribe({ id: 'acceptable_ads', del: false });
      // display the Yes/No buttons
      $('.afterDisableAA input')
        .prop('disabled', false);
      $('.afterDisableAA')
        .removeClass('afterDisableAA');
    });

  // if the user clicks a radio button
  $('#step_update_aa_no')
    .click(() => {
      $('#step_update_aa').children().remove();
      const newSpan = document.createElement('span');
      newSpan.setAttribute('class', 'answer');
      newSpan.setAttribute('chosen', 'no');
      const textNode = document.createTextNode(translate('no'));
      newSpan.appendChild(textNode);
      $('#step_update_aa').append(newSpan);
      $('#checkupdate')
        .text(translate('aamessageadreport'));
      $('#checkupdatelink')
        .text(translate('aalinkadreport'));
      $('#checkupdatelink_DIV')
        .fadeIn()
        .css('display', 'block');
    });

  $('#step_update_aa_yes')
    .click(() => {
      $('#step_update_aa').children().remove();
      const newSpan = document.createElement('span');
      newSpan.setAttribute('class', 'answer');
      newSpan.setAttribute('chosen', 'yes');
      const textNode = document.createTextNode(translate('yes'));
      newSpan.appendChild(textNode);
      $('#step_update_aa').append(newSpan);
      $('#step_disable_extensions_DIV')
        .fadeIn()
        .css('display', 'block');
    });

  // STEP 4: disable all extensions

  // Code for displaying the div is in the $function() that contains localizePage()
  // after user disables all extensions except for AdBlock
  // if the user clicks a radio button
  $('#step_disable_extensions_no').click(() => {
    $('#step_disable_extensions').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'no');
    const textNode = document.createTextNode(translate('no'));
    newSpan.appendChild(textNode);
    $('#step_disable_extensions').append(newSpan);
    $('#checkupdate').text(translate('reenableadsonebyone'));
  });

  $('#step_disable_extensions_yes').click(() => {
    $('#step_disable_extensions').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'yes');
    const textNode = document.createTextNode(translate('yes'));
    newSpan.appendChild(textNode);
    $('#step_disable_extensions').append(newSpan);
    $('#step_language_DIV').fadeIn().css('display', 'block');
  });

  // STEP 5: language
  // if the user clicks an item
  let contact = '';
  $('#step_language_lang').change(function stepLanguageLangChange() {
    const selected = $('#step_language_lang option:selected');
    $('#step_language').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    const textNode = document.createTextNode(selected.text());
    newSpan.appendChild(textNode);
    $('#step_language').append(newSpan);
    $('#step_language span').attr('chosen', selected.attr('i18n'));
    if (selected.text() === translate('other')) {
      if (!$('#checkupdate').get(0).firstChild) {
        log('returning, no first child found', $(this).attr('i18n'));
        return;
      }
      if (!$('#checkupdate').get(0).lastChild) {
        log('returning, no last child found', $(this).attr('i18n'));
        return;
      }
      const rawMessageText = translate('nodefaultfilter1');
      const messageSplit = splitMessageWithReplacementText(rawMessageText);
      $('#checkupdate').get(0).firstChild.nodeValue = messageSplit.anchorPrefixText;
      $('#checkupdate').get(0).lastChild.nodeValue = messageSplit.anchorPostfixText;
      $('#checkupdatelink').text(messageSplit.anchorText).attr('href', 'https://adblockplus.org/en/subscriptions');
      return;
    }
    const requiredLists = selected.attr('value').split(';');
    for (let i = 0; i < requiredLists.length - 1; i += 1) {
      if (unsubscribedDefaultFilters[requiredLists[i]]) {
        $('#checkupdate').text(translate('retryaftersubscribe', [translate(`filter${requiredLists[i]}`)]));
        return;
      }
    }


    contact = requiredLists[requiredLists.length - 1];

    $('#step_chrome_DIV').fadeIn().css('display', 'block');
    $('#checkinchrome').text(translate('ff_checkinchrometitle'));
  });

  // STEP 6: also in Chrome

  // If the user clicks a radio button
  $('#step_other_browser_yes').click(function stepOtherBrowserYesClick() {
    $('#step_other_browser').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'yes');
    const textNode = document.createTextNode(translate('yes'));
    newSpan.appendChild(textNode);
    $('#step_other_browser').append(newSpan);

    if (/^mailto:/.test(contact)) {
      contact = contact.replace(' at ', '@');
    }

    if (!$('#checkupdate').get(0).firstChild) {
      log('returning, no first child found', $(this).attr('i18n'));
      return;
    }
    if (!$('#checkupdate').get(0).lastChild) {
      log('returning, no last child found', $(this).attr('i18n'));
      return;
    }
    const rawMessageText = translate('reportfilterlistproblem');
    const messageSplit = splitMessageWithReplacementText(rawMessageText);
    $('#checkupdate').get(0).firstChild.nodeValue = messageSplit.anchorPrefixText;
    $('#checkupdate').get(0).lastChild.nodeValue = messageSplit.anchorPostfixText;
    $('#checkupdatelink').prop('href', contact);
    $('#checkupdatelink').text(contact.replace(/^mailto:/, ''));
    $('#privacy').show();
  });

  $('#step_other_browser_no').click(() => {
    $('#step_other_browser').children().remove();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'no');
    const textNode = document.createTextNode(translate('no'));
    newSpan.appendChild(textNode);
    $('#step_other_browser').append(newSpan);
    $('#step_report_DIV')
      .fadeIn()
      .css('display', 'block');
    if (debugInfo) {
      $('#debug-info')
        .val(createReadableReport({
          debug: debugInfo,
        }));
    }
  });

  $('#step_other_browser_wontcheck').click(() => {
    $('#step_other_browser_yes').click();
    const newSpan = document.createElement('span');
    newSpan.setAttribute('class', 'answer');
    newSpan.setAttribute('chosen', 'wont_check');
    const textNode = document.createTextNode(translate('refusetocheck'));
    newSpan.appendChild(textNode);
    $('#step_other_browser').append(newSpan);
  });

  // STEP 7: Ad Report
  $('#step_report_submit')
    .click(() => {
      $('#step_report_submit').prop('disabled', false);
      sendReport();
    });
});

